# Define your custom views here or overwrite the default views. Default
# CRUD operations are generated by the ringo frameworkd.
from ringo.lib.form import get_form_config_from_file
from efa.views.report import prepare_report
from efa.lib.report import (
    get_participants,
    create_virtual_participation_list,
)

def durchfuehrungsbericht(request, program, clazz, evaluate, recipe,
                          validation, derive_indicators, indicators,
                          filters, getvalidate=False):
    formconfig = get_form_config_from_file("efa", "reports.xml", "durchfuehrungsbericht")
    result = prepare_report(request, formconfig)
    result["out"] = None
    result["indicators"] = indicators
    result["program"] = program
    form = result.get("form")
    if getvalidate:
        form.validate(request.GET)
    if (request.POST or getvalidate) and not form.has_errors():
        start_date = form.data["start_date"]
        end_date = form.data["end_date"]
        filter_type = form.data["filter_type"]
        foerderphase = form.data.get("foerderphase")

        participants = get_participants(request, clazz)
        participations = create_virtual_participation_list(participants,
                                                           recipe,
                                                           validation,
                                                           program, derive_indicators)
        oi_fields = [i[0] for i in indicators["OI"]]
        ri_fields = [i[0] for i in indicators["RI"]]

        result["out"] = evaluate(participations, start_date, end_date, filter_type,
                                 oi_fields, ri_fields, filters,
                                 foerderphase=foerderphase)
        return result
    else:
        return result


def sachbericht(request, program, clazz, evaluate, recipe,
                validation, derive_indicators, indicators,
                filters, getvalidate=False):
    formconfig = get_form_config_from_file("efa", "reports.xml", "sachbericht")
    result = prepare_report(request, formconfig)
    result["out"] = None
    result["indicators"] = indicators
    result["program"] = program
    form = result.get("form")
    if getvalidate:
        form.validate(request.GET)
    if (request.POST or getvalidate) and not form.has_errors():
        start_date = form.data["start_date"]
        end_date = form.data["end_date"]
        filter_type = 4  # Start or End in timeframe

        participants = get_participants(request, clazz)
        participations = create_virtual_participation_list(participants,
                                                           recipe,
                                                           validation,
                                                           program, derive_indicators)
        oi_fields = [i[0] for i in indicators["OI"]]
        ri_fields = [i[0] for i in indicators["RI"]]

        result["out"] = evaluate(participations, start_date, end_date, filter_type,
                                 oi_fields, ri_fields, filters)
        return result
    else:
        return result


def evaluate(request,
             program,
             clazz,
             evaluate,
             recipe,
             validation,
             derive_indicators,
             indicators,
             filters,
             getvalidate=False):
    """View to generate the Report. This view will handle request in the
    webapplication and builds the correct form and converts the
    formdate.

    :request: Current request
    :programm: Name of the program e.g "waskiq".
    :clazz: Class of the Participation
    :evalute: Callable which actually does the evaluation.
    :recipe: Callable used to build the virtual participations.
    :validation: Callable used to to the validation on the
    participations. Can be None.
    :derive_indicators: Callable used to get the attributes used in the
    report from the participation. e.g the OI and RI.
    :indicators: Dictionary defining the indicators which are about to
    be evaluated
    :filters: Filters are used to split the result of the report by
    given cirteria e.g gender.
    """
    return sachbericht(request, program, clazz, evaluate, recipe, validation,
                       derive_indicators, indicators, filters, getvalidate)
